/*
 * $Id: hash.c,v 1.1.1.1 2002/02/23 02:47:02 kos Exp $
 *
 * TODO
 *  No data with the same key -> implement replace
 *  Reorg of the linked lists according to popularity
 *  leay's method of dynamic expansion/contraction based on demand
 *  abstract away the comparison function
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hash.h"
#include "xmalloc.h"

hash_t *
hash_new(int buckets) {
  hash_t *hash;
  int i;
  
  hash = xcalloc(1, sizeof(hash_t));
  hash->mode_replace = DEF_MODE_REPLACE;
  hash->nbuckets = buckets?buckets:DEF_HASH_NBUCKETS;
  hash->buckets = xcalloc(hash->nbuckets, sizeof(bucket_t *));

  for(i=0; i<hash->nbuckets; i++)
    hash->buckets[i] = (bucket_t *)NULL;
  
  return hash;
}

/* XXX */
void
hash_destroy(hash_t *h) {  
  int i;
  
  for(i=0; i<h->nbuckets; i++) {
  }
  
  xfree(h);
  return;
}

int
hash_func(hash_t *h, char *key) {
  int res=0;
  char *skey;

  skey = key;
  while(*key)
    res += *key++;
  res %= h->nbuckets;
  return res;
}

int
hash_exists(hash_t *h, char *key) {
  int p;
  bucket_t *b;
  
  p = hash_func(h, key);
  for(b=h->buckets[p]; b; b=b->next) {
    if(!strcmp(key, b->key))
      return 1;
  }
  return 0;
}

int
hash_add(hash_t *h, char *key, void *b, unsigned int l) {
  int p;
  bucket_t *new;
  
  p = hash_func(h, key);
  new = xcalloc(1, sizeof(bucket_t));

  new->next = h->buckets[p];
  new->prev = (bucket_t *)NULL;
  new->key = xstrdup(key);
  new->datap = xmalloc(l);
  new->datal = l;
  memcpy(new->datap, b, l);

  h->buckets[p] = new;
  return 0;
}

int
hash_nelem(hash_t *h) {
  int tot=0;
  int i;
  bucket_t *b;
  
  for(i=0; i<h->nbuckets; i++)
    for(b=h->buckets[i]; b; b=b->next,tot++)
      ;
  return tot;    
}

int
hash_index(hash_t *h, int index, char *p, int l) {
  int i;
  int c=0;
  bucket_t *b;

  for(i=0; i<h->nbuckets; i++) {
    for(b=h->buckets[i]; b; b=b->next,c++) {
      if(c == index) {
	strncpy(p, b->key, l);
	return 0;
      }
    }
  }
  return 1;
}

int
hash_get(hash_t *h, char *key, void **userb, unsigned int *userl) {
  int p;
  bucket_t *b;

  p = hash_func(h, key);
  for(b=h->buckets[p]; b; b=b->next) {
    if(!strcmp(key, b->key)) {
      *userb = b->datap;
      *userl = b->datal;
      return 0;
    }
  }
  return 1;
}

int
hash_delete(hash_t *h, char *key) {
  int p;
  bucket_t *b;

  p = hash_func(h, key);
  for(b=h->buckets[p]; b; b=b->next) {
    if(!strcmp(key, b->key)) {
      xfree(b->datap);
      if(b->prev)
	b->prev->next = b->next;
      if(b->next)
	b->next->prev = b->prev;
      xfree(b);
      return 0;
    }
  }
  return 1;
}

void
hash_dump_all(hash_t *h) {
  bucket_t *b;
  int i;

  for(i=0; i<h->nbuckets; i++) {
    printf("%d: ", i);
    for(b=h->buckets[i]; b; b=b->next) {
      printf("%s ", b->key);
    }
    printf("\n");
  }
}

#ifdef DEBUG_HASH

int
main(int argc, char **argv) {
  hash_t *h;
  char lhs[1024];
  char rhs[1024];
  int i;
  
  if(!(h = hash_new(12))) {
    printf("hash_init\n");
    exit(1);
  }

  for(i=0; i<19; i++) {
    snprintf(lhs, 1024, "key%d", i);
    snprintf(rhs, 1024, "disis data %d", i+144);
    elem.p = rhs;
    elem.l = strlen(rhs);
    hash_add(h, lhs, &elem);
  }

  hash_dump(h);  

  printf("%d\n", hash_exists(h, "key2"));
  printf("%d\n", hash_exists(h, "key2aa"));
  printf("%d\n", hash_exists(h, "key2vv"));

  for(i=0; i<hash_nelem(h); i++) {
    memset(lhs, '\0', 1024);
    hash_index(h, i, lhs, 1024);
    hash_get(h, lhs, &elem);
    printf("%d: key %s, data %s\n", i, lhs, elem.p);
  }
  
  hash_destroy(h);
  exit(0);
}

#endif
