/*
 * $Id: nldump.c,v 1.1.1.1 2002/02/23 02:47:02 kos Exp $
 *
 * Just a simple debugging tool for prism2 based cards.
 * The interface is bizarre..
 * 
 * output seems to be some sort of link-level header + radio
 * info (0x90) plus the 802.11 frame.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#include <signal.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <sys/socket.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <asm/types.h>
#include <linux/netlink.h>

#include "llog.h"

#define DEF_LINELEN 1024
#define MAX_PKTSIZE 3072
#define MCAST_GRP_SNIFF 0x00000002

int interrupted = 0;

void usage(char *argv0, int code) {
  fprintf(stderr, "Usage: %s <options>\n", argv0);
  fprintf(stderr, "\t-i <interface name>\n");
  fprintf(stderr, "\t-v <verbose level>\n");
  exit(code);
}

void
signal_handler(int sig) {
  switch(sig){
  case SIGINT:
    interrupted = 1;
  }
}

int
do_nl_read(char *device, int snaplen, char *filter) {
  fd_set fds;
  struct timeval tm;
  unsigned int count;
  int fd;
  struct sockaddr_nl sanl;
  unsigned char pkt[MAX_PKTSIZE];
  unsigned int pktlen;

  count = 0;

  if((fd = socket(PF_NETLINK, SOCK_RAW, MCAST_GRP_SNIFF)) < 0) {
    llog(0, "socket(PF_NETLINK: %s\n", strerror(errno));
    return 1;
  }

  memset((void *)&sanl, 0, sizeof(struct sockaddr_nl));
  sanl.nl_family = (sa_family_t) PF_NETLINK;
  sanl.nl_pid = (unsigned int) getpid ();
  sanl.nl_groups = MCAST_GRP_SNIFF;
  
  if(bind(fd, (struct sockaddr *)&sanl, sizeof(struct sockaddr_nl)) < 0) {
    llog(0, "bind(PF_NETLINK): %s\n", strerror(errno));
    return 2;
  }

  while(1) {
    if(interrupted)
      break;
    
    FD_ZERO(&fds);
    FD_SET(fd, &fds);
    tm.tv_sec = 0;
    tm.tv_usec = 500;
    if(select(fd+1, &fds, NULL, NULL, NULL) < 0)
      break;
    
    if(!FD_ISSET(fd, &fds)) {
      continue;
    }
  
    if((pktlen = recv(fd, pkt, MAX_PKTSIZE, 0)) < 0) {
      llog(0, "recvfrom(PF_NETLINK): %s\n", strerror(errno));
      return 3;
    }

    llog(1, "> frame %d, length %u (0x%x)\n",
	 count, pktlen, pktlen);
    llog_hexdump(1, pkt, pktlen);
    count++;
  }

  close(fd);
  return 0;
}

int
main(int argc, char **argv) {
  char c;
  int snaplen;
  char device[DEF_LINELEN];
  int verbose;

  snaplen = 2500;
  verbose = 1;
  device[0] = '\0';

  while((c = getopt(argc, argv, "i:v:f:")) != EOF) {
    switch(c) {
    case 'i':
      strncpy(device, optarg, DEF_LINELEN);
      break;
    case 'v':
      verbose = atoi(optarg);
      break;
    case '?':
      usage(argv[0], 1);
      break;
    }
  }

  llog_init(LLOG_STDERR);
  llog_level(verbose);
  signal(SIGINT, signal_handler);
  
  do_nl_read(device, snaplen, "");

  llog_close();
  exit(0);
}

