/*
 * $Id: ssidsniff.h,v 1.4 2002/03/03 22:11:04 kos Exp $
 *
 */

#ifndef _SSIDSNIFF_H
#define _SSIDSNIFF_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

#if HAVE_LINUX_WIRELESS_H
#include <sys/ioctl.h>
#include <netdb.h>
#include <linux/socket.h>
#include <sys/socket.h>
#include <linux/wireless.h>
#endif

#include <pcap.h>

#include "xpaudio.h"

#define DEF_LINELEN 1024
#define DEF_GEIGER_L 128

#define MGT_ASSOC_REQ        0x00       /* Management - association request        */
#define MGT_ASSOC_RESP       0x01       /* Management - association response       */
#define MGT_REASSOC_REQ      0x02       /* Management - reassociation request      */
#define MGT_REASSOC_RESP     0x03       /* Management - reassociation response     */
#define MGT_PROBE_REQ        0x04       /* Management - Probe request              */
#define MGT_PROBE_RESP       0x05       /* Management - Probe response             */
#define MGT_BEACON           0x08       /* Management - Beacon frame               */
#define MGT_ATIM             0x09       /* Management - ATIM                       */
#define MGT_DISASS           0x0A       /* Management - Disassociation             */
#define MGT_AUTHENTICATION   0x0B       /* Management - Authentication             */
#define MGT_DEAUTHENTICATION 0x0C       /* Management - Deauthentication           */

typedef struct frame_control_t {
  unsigned version:2;
  unsigned type:2;
  unsigned subtype:4;
  unsigned to_ds:1;
  unsigned from_ds:1;
  unsigned more_frag:1;
  unsigned retry:1;
  unsigned powermgmt:1;
  unsigned more_data:1;
  unsigned wep:1;
  unsigned order:1;
} frame_control_t;

typedef struct f80211_mac_t {
  unsigned char fc[2];
  unsigned char id[2];
  unsigned char address1[6];
  unsigned char address2[6];
  unsigned char address3[6];
  unsigned char seq[2];
} f80211_mac_t;

#define IEEE80211_CAPINFO_ESS                   0x0001
#define IEEE80211_CAPINFO_IBSS                  0x0002
#define IEEE80211_CAPINFO_CF_POLLABLE           0x0004
#define IEEE80211_CAPINFO_CF_POLLREQ            0x0008
#define IEEE80211_CAPINFO_PRIVACY               0x0010
#define IEEE80211_CAPINFO_SHORT_PREAMBLE        0x0020
#define IEEE80211_CAPINFO_PBCC                  0x0040
#define IEEE80211_CAPINFO_CHNL_AGILITY          0x0080

typedef struct beacon_header_t {
  unsigned char timestamp[8];
  unsigned char interval[2];
  unsigned char capability[2];
} beacon_header_t;

typedef struct packetinfo_t {
  unsigned char ssid[DEF_LINELEN];  
  unsigned char mac[DEF_LINELEN];
  unsigned char bss[DEF_LINELEN];
  unsigned char dest[DEF_LINELEN];
  int channel;
  int type;
  int subtype;
  int wepcapable;
  int wepused;
  int ap;
  time_t timestamp;
  unsigned int serial;
  unsigned char bitrate[DEF_LINELEN];
  unsigned char quality[DEF_LINELEN];
  unsigned char level[DEF_LINELEN];
  unsigned char noise[DEF_LINELEN];
} packetinfo_t;

typedef struct infosink_t {
  unsigned int total;
  unsigned int beacon;
  unsigned int data;
  unsigned int martian;
  unsigned int wep;
} infosink_t;

typedef enum {live, file} sniff_mode_t;
typedef enum {simple, machine} runlog_mode_t;

typedef struct ssidsniff_t {
  sniff_mode_t mode;
  int mode_geiger;
  int strip_80211;
  int interactive;
  int maxcount; 
  char device[PCAP_ERRBUF_SIZE];
  int pcap_snaplen;
  int pcap_promiscuous;
  char pcap_filter[DEF_LINELEN];
  char pcap_datadump[DEF_LINELEN];
  struct bpf_program pcap_fcode;
  unsigned int pcap_if_net;
  unsigned int pcap_if_mask;
  char pcap_ebuf[PCAP_ERRBUF_SIZE];
  pcap_t *pd;
  pcap_dumper_t *pd_dump;
  int pcap_fd;
  xpaudio_t *xpa;
  FILE *runlog;
  runlog_mode_t runlog_mode;
} ssidsniff_t;

#endif /* _SSIDSNIFF_H */
